pub struct StackLevel<ET: EngineTypes> {
pub group_type: GroupType,
pub aftergroup: Vec<ET::Token>,
/* private fields */
}Expand description
A level of the StateStack, to be rolled back when a group ends
Fields
group_type: GroupTypeThe type of the group
aftergroup: Vec<ET::Token>The \aftergroup Tokens to be inserted when the group ends