pub struct StateStack<ET: EngineTypes> {
pub stack: Vec<StackLevel<ET>>,
/* private fields */
}Expand description
A stack of StateChanges, to be rolled back when a group ends
Fields
stack: Vec<StackLevel<ET>>The stack of StackLevels
Implementations
Sourceimpl<ET: EngineTypes> StateStack<ET>
impl<ET: EngineTypes> StateStack<ET>
Sourcepub fn push(&mut self, group_type: GroupType)
pub fn push(&mut self, group_type: GroupType)
Push a new stack level onto the stack with the given GroupType, as a new group begins
Sourcepub fn pop(&mut self) -> (GroupType, Vec<ET::Token>, ChangeIter<'_, ET>)
pub fn pop(&mut self) -> (GroupType, Vec<ET::Token>, ChangeIter<'_, ET>)
Pop a stack level from the stack, as a group ends. Returns the GroupType, the \aftergroup Tokens,
and an iterator over the StateChanges to be rolled back.
Sourcepub fn add_change_globally(&mut self, change: StateChange<ET>)
pub fn add_change_globally(&mut self, change: StateChange<ET>)
Register a global state change, never to be rolled back; if there is a stack level, remove any equivalent previous changes
Sourcepub fn add_change_locally(&mut self, change: StateChange<ET>)
pub fn add_change_locally(&mut self, change: StateChange<ET>)
Register a local state change, to be rolled back when the current group ends